@@ -28,21 +28,33 @@
 #' select_positions(mtcars, mpg:drat)
 #'
 #' @noRd
-select_positions <- function(.data, ..., group_pos = FALSE) {
-  cols <- eval(substitute(alist(...)))
-  data_names <- colnames(.data)
-  select_env$.col_names <- data_names
-  on.exit(rm(list = ".col_names", envir = select_env))
+select_positions <- function(.data, ..., .group_pos = FALSE) {
+  cols <- dotdotdot(...)
+  select_env$setup(.data = .data, calling_frame = parent.frame(2L))
+  on.exit(select_env$clean(), add = TRUE)
+  data_names <- select_env$get_colnames()
   pos <- unlist(lapply(cols, eval_expr))
-  if (isTRUE(group_pos)) {
+  col_len <- select_env$get_ncol()
+  if (any(pos > col_len)) {
+    oor <- pos[which(pos > col_len)]
+    oor_len <- length(oor)
+    stop(
+      "Location", if (oor_len > 1) "s " else " ", collapse_to_sentence(oor),
+      if (oor_len > 1) " don't " else " doesn't ", "exist. There are only ", col_len, " columns."
+    )
+  }
+  if (isTRUE(.group_pos)) {
     groups <- get_groups(.data)
     missing_groups <- !(groups %in% cols)
     if (any(missing_groups)) {
-      message("Adding missing grouping variables: `", paste(groups[missing_groups], collapse = "`, `"), "`")
-      pos <- c(match(groups[missing_groups], data_names), pos)
+      sel_missing <- groups[missing_groups]
+      message("Adding missing grouping variables: `", paste(sel_missing, collapse = "`, `"), "`")
+      readd <- match(sel_missing, data_names)
+      if (length(names(cols)) > 0L) names(readd) <- data_names[readd]
+      pos <- c(readd, pos)
     }
   }
-  unique(pos)
+  pos[!duplicated(pos)]
 }
 
 eval_expr <- function(x) {
@@ -59,13 +71,27 @@ eval_expr <- function(x) {
 }
 
 select_char <- function(expr) {
-  pos <- match(expr, select_env$.col_names)
+  pos <- match(expr, select_env$get_colnames())
   if (is.na(pos)) stop("Column `", expr, "` does not exist")
   pos
 }
 
 select_symbol <- function(expr) {
-  select_char(as.character(expr))
+  expr_name <- as.character(expr)
+  if (grepl("^is\\.", expr_name) && is_function(expr)) {
+    stop(
+      "Predicate functions must be wrapped in `where()`.\n\n",
+      sprintf("  data %%>%% select(where(%s))", expr_name)
+    )
+  }
+  res <- try(select_char(as.character(expr)), silent = TRUE)
+  if (inherits(res, "try-error")) {
+    res <- tryCatch(
+      unlist(lapply(eval(expr, envir = select_env$calling_frame), eval_expr)),
+      error = function(e) stop("Column ", expr, " does not exist.")
+    )
+  }
+  res
 }
 
 eval_call <- function(x) {
@@ -117,5 +143,17 @@ select_bracket <- function(expr) {
 }
 
 select_context <- function(expr) {
-  eval(expr, envir = context$.data)
+  eval(expr, envir = select_env$.data)
 }
+
+select_env <- new.env()
+select_env$setup <- function(.data, calling_frame) {
+  select_env$.data <- .data
+  select_env$calling_frame <- calling_frame
+}
+select_env$clean <- function() {
+  rm(list = c(".data", "calling_frame"), envir = select_env)
+}
+select_env$get_colnames <- function() colnames(select_env$.data)
+select_env$get_nrow <- function() nrow(select_env$.data)
+select_env$get_ncol <- function() ncol(select_env$.data)
