@@ -6,8 +6,11 @@
 #' `summarise()` and `summarize()` are synonyms.
 #'
 #' @param .data A `data.frame`.
-#' @param ... Name-value pairs of summary functions. The name will be the name of the variable in the result. The value
-#' should be an expression that returns a single value, e.g. `min(x)`.
+#' @param ... Name-value pairs of summary functions. The name will be the name of the variable in the result.
+#'
+#' The value can be:
+#' * A vector of length `1`, e.g. `min(x)`, `n()`, or `sum(is.na(y))`.
+#' * A vector of length `n`, e.g. `quantile()`.
 #'
 #' @examples
 #' summarise(mtcars, mean(mpg))
@@ -23,25 +26,31 @@ summarise <- function(.data, ...) {
 
 #' @export
 summarise.default <- function(.data, ...) {
-  fns <- vapply(substitute(...()), deparse, NA_character_)
-  context$.data <- .data
-  on.exit(rm(.data, envir = context))
-  if (has_groups(.data)) {
-    group <- unique(.data[, get_groups(.data), drop = FALSE])
-    if (nrow(group) == 0L) return(NULL)
+  fns <- dotdotdot(...)
+  context$setup(.data)
+  on.exit(context$clean(), add = TRUE)
+  groups_exist <- has_groups(context$.data)
+  if (groups_exist) {
+    group <- unique(context$.data[, get_groups(context$.data), drop = FALSE])
   }
-  res <- lapply(fns, function(x) do.call(with, list(.data, str2lang(x))))
+  res <- lapply(
+    fns,
+    function(x) {
+      x_res <- do.call(with, list(context$.data, x))
+      if (is.list(x_res)) I(x_res) else x_res
+    }
+  )
   res <- as.data.frame(res)
   fn_names <- names(fns)
   colnames(res) <- if (is.null(fn_names)) fns else fn_names
-  if (has_groups(.data)) res <- cbind(group, res)
+  if (groups_exist) res <- cbind(group, res, row.names = NULL)
   res
 }
 
 #' @export
 summarise.grouped_data <- function(.data, ...) {
   groups <- get_groups(.data)
-  res <- apply_grouped_function(.data, "summarise", ...)
+  res <- apply_grouped_function("summarise", .data, drop = TRUE, ...)
   res <- res[do.call(order, lapply(groups, function(x) res[, x])), ]
   rownames(res) <- NULL
   res
