@@ -7,7 +7,7 @@ table2 <- table1[c(1, 3, 5), ]
 
 expect_equal(
   {
-    invisible(capture.output(res <- table1 %>% anti_join(table2)))
+    invisible(suppressMessages(res <- table1 %>% anti_join(table2)))
     res
   },
   data.frame(id = c(2, 4), animal = c("dog", "parrot"), stringsAsFactors = FALSE),
@@ -16,7 +16,7 @@ expect_equal(
 
 expect_equal(
   {
-    invisible(capture.output(res <- table1 %>% semi_join(table2)))
+    invisible(suppressMessages(res <- table1 %>% semi_join(table2)))
     res
   },
   data.frame(id = c(1L, 3L, 5L), animal = c("cat", "hamster", "horse"), stringsAsFactors = FALSE),
